<?php
/* --------------------------------------------------------------
   CustomerGroupFactory.inc.php 2017-09-27
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class CustomerGroupFactory
 *
 * @category   System
 * @package    CustomerGroup
 * @subpackage Factories
 */
class CustomerGroupFactory
{
	/**
	 * @var \CustomerGroupRepositoryInterface
	 */
	private $repository;
	
	
	/**
	 * CustomerGroupFactory constructor.
	 *
	 * @param \CustomerGroupRepositoryInterface $repository
	 */
	public function __construct(CustomerGroupRepositoryInterface $repository)
	{
		$this->repository = $repository;
	}
	
	
	/**
	 * Returns new instances of customer group entities.
	 *
	 * @return \CustomerGroup
	 */
	public function createEntity()
	{
		return new CustomerGroup($this->repository, new EditableKeyValueCollection([]));
	}
	
	
	/**
	 * Returns a new instance of customer group settings.
	 *
	 * @param \BoolType $statusPublic
	 * @param \BoolType $statusOtDiscount
	 * @param \BoolType $statusGraduatedPrices
	 * @param \BoolType $statusShowPrice
	 * @param \BoolType $statusShowPriceTax
	 * @param \BoolType $statusAddTaxOt
	 * @param \BoolType $statusDiscountAttributes
	 * @param \BoolType $statusFsk18
	 * @param \BoolType $statusFsk18Display
	 * @param \BoolType $statusWriteReviews
	 * @param \BoolType $statusReadReviews
	 *
	 * @return \CustomerGroupSettings
	 */
	public function createSettings(BoolType $statusPublic,
	                               BoolType $statusOtDiscount,
	                               BoolType $statusGraduatedPrices,
	                               BoolType $statusShowPrice,
	                               BoolType $statusShowPriceTax,
	                               BoolType $statusAddTaxOt,
	                               BoolType $statusDiscountAttributes,
	                               BoolType $statusFsk18,
	                               BoolType $statusFsk18Display,
	                               BoolType $statusWriteReviews,
	                               BoolType $statusReadReviews)
	{
		return new CustomerGroupSettings($statusPublic, $statusOtDiscount, $statusGraduatedPrices, $statusShowPrice,
		                                 $statusShowPriceTax, $statusAddTaxOt, $statusDiscountAttributes, $statusFsk18,
		                                 $statusFsk18Display, $statusWriteReviews, $statusReadReviews);
	}
	
	
	public function createConfigurations(DecimalType $discount,
	                                     DecimalType $otDiscount,
	                                     DecimalType $minOrder = null,
	                                     DecimalType $maxOrder = null,
	                                     array $unallowedPaymentModules = [],
	                                     array $unallowedShippingModules = [])
	{
		return new CustomerGroupConfigurations($discount, $otDiscount, $minOrder, $maxOrder, $unallowedPaymentModules,
		                                       $unallowedShippingModules);
	}
	
	
	/**
	 * Returns new instances of customer group collections.
	 *
	 * @return \CustomerGroupCollection
	 */
	public function createCollection()
	{
		return new CustomerGroupCollection();
	}
}